@@ -10,7 +10,11 @@
 #define XTENSOR_R_CONFIG_HPP
 
 #define XTENSOR_R_VERSION_MAJOR 0
-#define XTENSOR_R_VERSION_MINOR 8
-#define XTENSOR_R_VERSION_PATCH 1
+#define XTENSOR_R_VERSION_MINOR 9
+#define XTENSOR_R_VERSION_PATCH 0
+
+#ifndef XTENSOR_WARN_ON_COERCE
+#define XTENSOR_WARN_ON_COERCE 1
+#endif
 
 #endif
