@@ -18,6 +18,8 @@
 
 #include "xarray.hpp"
 #include "xtensor.hpp"
+#include "xfixed.hpp"
+#include "xbuffer_adaptor.hpp"
 
 namespace xt
 {
@@ -34,6 +36,15 @@ namespace xt
 
         template <class C>
         using array_size = array_size_impl<std::decay_t<C>>;
+
+        template <class P>
+        struct default_allocator_for_ptr
+        {
+            using type = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>;
+        };
+
+        template <class P>
+        using default_allocator_for_ptr_t = typename default_allocator_for_ptr<P>::type;
     }
 
     /**************************
@@ -48,10 +59,22 @@ namespace xt
      * @param l the layout_type of the xarray_adaptor
      */
     template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, class SC,
-              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int> = 0>
+              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int> = 0,
+              typename std::enable_if_t<!std::is_pointer<C>::value, int> = 0>
     xarray_adaptor<xtl::closure_type_t<C>, L, std::decay_t<SC>>
     adapt(C&& container, const SC& shape, layout_type l = L);
 
+    /**
+     * Constructs an non-owning xarray_adaptor from a pointer with the specified shape and layout.
+     * @param pointer the container to adapt
+     * @param shape the shape of the xarray_adaptor
+     * @param l the layout_type of the xarray_adaptor
+     */
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, class SC,
+              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int> = 0,
+              typename std::enable_if_t<std::is_pointer<C>::value, int> = 0>
+    inline auto adapt(C&& pointer, const SC& shape, layout_type l = L);
+
     /**
      * Constructs an xarray_adaptor of the given stl-like container,
      * with the specified shape and strides.
@@ -71,12 +94,12 @@ namespace xt
      * @param pointer the pointer to the beginning of the dynamic array
      * @param size the size of the dynamic array
      * @param ownership indicates whether the adaptor takes ownership of the array.
-     *        Possible values are ``no_ownerhsip()`` or ``acquire_ownership()``
+     *        Possible values are ``no_ownership()`` or ``acquire_ownership()``
      * @param shape the shape of the xarray_adaptor
      * @param l the layout_type of the xarray_adaptor
      * @param alloc the allocator used for allocating / deallocating the dynamic array
      */
-    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class SC, class A = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>,
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class SC, class A = detail::default_allocator_for_ptr_t<P>,
               typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int> = 0>
     xarray_adaptor<xbuffer_adaptor<xtl::closure_type_t<P>, O, A>, L, SC>
     adapt(P&& pointer, typename A::size_type size, O ownership, const SC& shape, layout_type l = L, const A& alloc = A());
@@ -87,12 +110,12 @@ namespace xt
      * @param pointer the pointer to the beginning of the dynamic array
      * @param size the size of the dynamic array
      * @param ownership indicates whether the adaptor takes ownership of the array.
-     *        Possible values are ``no_ownerhsip()`` or ``acquire_ownership()``
+     *        Possible values are ``no_ownership()`` or ``acquire_ownership()``
      * @param shape the shape of the xarray_adaptor
      * @param strides the strides of the xarray_adaptor
      * @param alloc the allocator used for allocating / deallocating the dynamic array
      */
-    template <class P, class O, class SC, class SS, class A = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>,
+    template <class P, class O, class SC, class SS, class A = detail::default_allocator_for_ptr_t<P>,
               typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int> = 0,
               typename std::enable_if_t<!std::is_same<layout_type, std::decay_t<SS>>::value, int> = 0>
     xarray_adaptor<xbuffer_adaptor<xtl::closure_type_t<P>, O, A>, layout_type::dynamic, std::decay_t<SC>>
@@ -120,10 +143,22 @@ namespace xt
      * @param l the layout_type of the xtensor_adaptor
      */
     template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, class SC,
-              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int> = 0>
+              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int> = 0,
+              typename std::enable_if_t<!std::is_pointer<C>::value, int> = 0>
     xtensor_adaptor<C, detail::array_size<SC>::value, L>
     adapt(C&& container, const SC& shape, layout_type l = L);
 
+    /**
+     * Constructs an non-owning xtensor_adaptor from a pointer with the specified shape and layout.
+     * @param pointer the pointer to adapt
+     * @param shape the shape of the xtensor_adaptor
+     * @param l the layout_type of the xtensor_adaptor
+     */
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, class SC,
+              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int> = 0,
+              typename std::enable_if_t<std::is_pointer<C>::value, int> = 0>
+    auto adapt(C&& pointer, const SC& shape, layout_type l = L);
+
     /**
      * Constructs an xtensor_adaptor of the given stl-like container,
      * with the specified shape and strides.
@@ -143,11 +178,11 @@ namespace xt
      * @param pointer the pointer to the beginning of the dynamic array
      * @param size the size of the dynamic array
      * @param ownership indicates whether the adaptor takes ownership of the array.
-     *        Possible values are ``no_ownerhsip()`` or ``acquire_ownership()``
+     *        Possible values are ``no_ownership()`` or ``acquire_ownership()``
      * @param l the layout_type of the xtensor_adaptor
      * @param alloc the allocator used for allocating / deallocating the dynamic array
      */
-    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class A = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>>
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class A = detail::default_allocator_for_ptr_t<P>>
     xtensor_adaptor<xbuffer_adaptor<xtl::closure_type_t<P>, O, A>, 1, L>
     adapt(P&& pointer, typename A::size_type size, O ownership, layout_type l = L, const A& alloc = A());
 
@@ -157,12 +192,12 @@ namespace xt
      * @param pointer the pointer to the beginning of the dynamic array
      * @param size the size of the dynamic array
      * @param ownership indicates whether the adaptor takes ownership of the array.
-     *        Possible values are ``no_ownerhsip()`` or ``acquire_ownership()``
+     *        Possible values are ``no_ownership()`` or ``acquire_ownership()``
      * @param shape the shape of the xtensor_adaptor
      * @param l the layout_type of the xtensor_adaptor
      * @param alloc the allocator used for allocating / deallocating the dynamic array
      */
-    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class SC, class A = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>,
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class P, class O, class SC, class A = detail::default_allocator_for_ptr_t<P>,
               typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int> = 0>
     xtensor_adaptor<xbuffer_adaptor<xtl::closure_type_t<P>, O, A>, detail::array_size<SC>::value, L>
     adapt(P&& pointer, typename A::size_type size, O ownership, const SC& shape, layout_type l = L, const A& alloc = A());
@@ -173,24 +208,36 @@ namespace xt
      * @param pointer the pointer to the beginning of the dynamic array
      * @param size the size of the dynamic array
      * @param ownership indicates whether the adaptor takes ownership of the array.
-     *        Possible values are ``no_ownerhsip()`` or ``acquire_ownership()``
+     *        Possible values are ``no_ownership()`` or ``acquire_ownership()``
      * @param shape the shape of the xtensor_adaptor
      * @param strides the strides of the xtensor_adaptor
      * @param alloc the allocator used for allocating / deallocating the dynamic array
      */
-    template <class P, class O, class SC, class SS, class A = std::allocator<std::remove_const_t<std::remove_pointer_t<std::remove_reference_t<P>>>>,
+    template <class P, class O, class SC, class SS, class A = detail::default_allocator_for_ptr_t<P>,
               typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int> = 0,
               typename std::enable_if_t<!std::is_same<layout_type, std::decay_t<SS>>::value, int> = 0>
     xtensor_adaptor<xbuffer_adaptor<xtl::closure_type_t<P>, O, A>, detail::array_size<SC>::value, layout_type::dynamic>
     adapt(P&& pointer, typename A::size_type size, O ownership, SC&& shape, SS&& strides, const A& alloc = A());
 
+    /**
+     * Constructs an non-owning xtensor_fixed_adaptor from a pointer with the
+     * specified shape and layout.
+     * @param pointer the pointer to adapt
+     * @param shape the shape of the xtensor_fixed_adaptor
+     * @param l the layout_type of the xtensor_fixed_adaptor
+     */
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, std::size_t... X,
+              typename std::enable_if_t<std::is_pointer<C>::value, int> = 0>
+    inline auto adapt(C&& ptr, const fixed_shape<X...>& /*shape*/);
+
     /*****************************************
      * xarray_adaptor builder implementation *
      *****************************************/
 
     // shape only - container version
     template <layout_type L, class C, class SC,
-              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int>>
+              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int>,
+              typename std::enable_if_t<!std::is_pointer<C>::value, int>>
     inline xarray_adaptor<xtl::closure_type_t<C>, L, std::decay_t<SC>>
     adapt(C&& container, const SC& shape, layout_type l)
     {
@@ -198,6 +245,17 @@ namespace xt
         return return_type(std::forward<C>(container), shape, l);
     }
 
+    template <layout_type L, class C, class SC,
+              typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int>,
+              typename std::enable_if_t<std::is_pointer<C>::value, int>>
+    inline auto adapt(C&& pointer, const SC& shape, layout_type l)
+    {
+        using buffer_type = xbuffer_adaptor<C, xt::no_ownership, detail::default_allocator_for_ptr_t<C>>;
+        using return_type = xarray_adaptor<buffer_type, L, std::decay_t<SC>>;
+        std::size_t size = compute_size(shape);
+        return return_type(buffer_type(pointer, size), shape, l);
+    }
+
     // shape and strides - container version
     template <class C, class SC, class SS,
               typename std::enable_if_t<!detail::is_array<std::decay_t<SC>>::value, int>,
@@ -207,8 +265,8 @@ namespace xt
     {
         using return_type = xarray_adaptor<xtl::closure_type_t<C>, layout_type::dynamic, std::decay_t<SC>>;
         return return_type(std::forward<C>(container),
-                           xtl::forward_sequence<typename return_type::inner_shape_type>(shape),
-                           xtl::forward_sequence<typename return_type::inner_strides_type>(strides));
+                           xtl::forward_sequence<typename return_type::inner_shape_type, SC>(shape),
+                           xtl::forward_sequence<typename return_type::inner_strides_type, SS>(strides));
     }
 
     // shape only - buffer version
@@ -234,8 +292,8 @@ namespace xt
         using return_type = xarray_adaptor<buffer_type, layout_type::dynamic, std::decay_t<SC>>;
         buffer_type buf(std::forward<P>(pointer), size, alloc);
         return return_type(std::move(buf),
-                           xtl::forward_sequence<typename return_type::inner_shape_type>(shape),
-                           xtl::forward_sequence<typename return_type::inner_strides_type>(strides));
+                           xtl::forward_sequence<typename return_type::inner_shape_type, SC>(shape),
+                           xtl::forward_sequence<typename return_type::inner_strides_type, SS>(strides));
     }
 
     /******************************************
@@ -254,7 +312,8 @@ namespace xt
 
     // shape only - container version
     template <layout_type L, class C, class SC,
-              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int>>
+              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int>,
+              typename std::enable_if_t<!std::is_pointer<C>::value, int>>
     inline xtensor_adaptor<C, detail::array_size<SC>::value, L>
     adapt(C&& container, const SC& shape, layout_type l)
     {
@@ -263,6 +322,42 @@ namespace xt
         return return_type(std::forward<C>(container), shape, l);
     }
 
+    template <layout_type L, class C, class SC,
+              typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int>,
+              typename std::enable_if_t<std::is_pointer<C>::value, int>>
+    inline auto adapt(C&& ptr, const SC& shape, layout_type l)
+    {
+        using buffer_type = xbuffer_adaptor<C, xt::no_ownership, detail::default_allocator_for_ptr_t<C>>;
+        constexpr std::size_t N = detail::array_size<SC>::value;
+        using return_type = xtensor_adaptor<buffer_type, N, L>;
+        return return_type(buffer_type(ptr, compute_size(shape)), shape, l);
+    }
+
+    template <layout_type L, class C, std::size_t... X,
+              typename std::enable_if_t<std::is_pointer<C>::value, int>>
+    inline auto adapt(C&& ptr, const fixed_shape<X...>& /*shape*/)
+    {
+        using buffer_type = xbuffer_adaptor<C, xt::no_ownership, detail::default_allocator_for_ptr_t<C>>;
+        using return_type = xfixed_adaptor<buffer_type, fixed_shape<X...>, L>;
+        return return_type(buffer_type(ptr, detail::fixed_compute_size<fixed_shape<X...>>::value));
+    }
+
+#ifndef X_OLD_CLANG
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C, class T,  std::size_t N>
+    inline auto adapt(C&& ptr, const T(&shape)[N])
+    {
+        using shape_type = std::array<std::size_t, N>;
+        return adapt(std::forward<C>(ptr), xtl::forward_sequence<shape_type, decltype(shape)>(shape));
+    }
+#else
+    template <layout_type L = XTENSOR_DEFAULT_LAYOUT, class C>
+    inline auto adapt(C&& ptr, std::initializer_list<std::size_t> shape)
+    {
+        using shape_type = xt::dynamic_shape<std::size_t>;
+        return adapt(std::forward<C>(ptr), xtl::forward_sequence<shape_type, decltype(shape)>(shape));
+    }
+#endif
+
     // shape and strides - container version
     template <class C, class SC, class SS,
               typename std::enable_if_t<detail::is_array<std::decay_t<SC>>::value, int>,
@@ -273,8 +368,8 @@ namespace xt
         constexpr std::size_t N = detail::array_size<SC>::value;
         using return_type = xtensor_adaptor<xtl::closure_type_t<C>, N, layout_type::dynamic>;
         return return_type(std::forward<C>(container),
-                           xtl::forward_sequence<typename return_type::inner_shape_type>(shape),
-                           xtl::forward_sequence<typename return_type::inner_strides_type>(strides));
+                           xtl::forward_sequence<typename return_type::inner_shape_type, SC>(shape),
+                           xtl::forward_sequence<typename return_type::inner_strides_type, SS>(strides));
     }
 
     // 1-D case - buffer version
@@ -314,8 +409,8 @@ namespace xt
         using return_type = xtensor_adaptor<buffer_type, N, layout_type::dynamic>;
         buffer_type buf(std::forward<P>(pointer), size, alloc);
         return return_type(std::move(buf),
-                           xtl::forward_sequence<typename return_type::inner_shape_type>(shape),
-                           xtl::forward_sequence<typename return_type::inner_strides_type>(strides));
+                           xtl::forward_sequence<typename return_type::inner_shape_type, SC>(shape),
+                           xtl::forward_sequence<typename return_type::inner_strides_type, SS>(strides));
     }
 }
 
