@@ -19,14 +19,12 @@
 
 #include "xlayout.hpp"
 #include "xshape.hpp"
+#include "xtensor_forward.hpp"
 #include "xutils.hpp"
 
 namespace xt
 {
 
-    template <class E>
-    class xshared_expression;
-
     /***************************
      * xexpression declaration *
      ***************************/
@@ -103,7 +101,7 @@ namespace xt
     //@}
 
     /* is_crtp_base_of<B, E>
-    * Resembles std::is_base_of, but adresses the problem of whether _some_ instatntiation
+    * Resembles std::is_base_of, but adresses the problem of whether _some_ instantiation
     * of a CRTP templated class B is a base of class E. A CRTP templated class is correctly
     * templated with the most derived type in the CRTP hierarchy. Using this assumption,
     * this implementation deals with either CRTP final classes (checks for inheritance
@@ -136,6 +134,31 @@ namespace xt
     template <class... E>
     using has_xexpression = xtl::disjunction<is_xexpression<E>...>;
 
+    /***********************
+     * evaluation_strategy *
+     ***********************/
+
+    namespace evaluation_strategy
+    {
+        struct base
+        {
+        };
+
+        struct immediate : base
+        {
+        };
+        
+        struct lazy : base
+        {
+        };
+        
+        /*
+        struct cached
+        {
+        };
+        */
+    }
+
     /************
      * xclosure *
      ************/
@@ -143,6 +166,9 @@ namespace xt
     template <class T>
     class xscalar;
 
+    template <class E>
+    class xshared_expression;
+
     template <class E, class EN = void>
     struct xclosure
     {
@@ -210,14 +236,41 @@ namespace xt
     template <class E>
     using xvalue_type_t = typename xvalue_type<E>::type;
 
+    /***********************************
+     * temporary_type_t implementation *
+     ***********************************/
+
+    namespace detail
+    {
+        template <class S>
+        struct xtype_for_shape
+        {
+            template <class T, layout_type L>
+            using type = xarray<T, L>;
+        };
+
+        template <template <class, std::size_t> class S, class X, std::size_t N>
+        struct xtype_for_shape<S<X, N>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor<T, N, L>;
+        };
+
+        template <template <std::size_t...> class S, std::size_t... X>
+        struct xtype_for_shape<S<X...>>
+        {
+            template <class T, layout_type L>
+            using type = xtensor_fixed<T, xshape<X...>, L>;
+        };
+
+        template <class T, class S, layout_type L>
+        using temporary_type_t = typename xtype_for_shape<S>::template type<T, L>;
+    }
+ 
     /*************************
      * expression tag system *
      *************************/
 
-    struct xscalar_expression_tag
-    {
-    };
-
     struct xtensor_expression_tag
     {
     };
@@ -226,26 +279,37 @@ namespace xt
     {
     };
 
-    namespace detail
+    namespace extension
     {
         template <class E, class = void_t<int>>
-        struct get_expression_tag
+        struct get_expression_tag_impl
         {
             using type = xtensor_expression_tag;
         };
 
         template <class E>
-        struct get_expression_tag<E, void_t<typename std::decay_t<E>::expression_tag>>
+        struct get_expression_tag_impl<E, void_t<typename std::decay_t<E>::expression_tag>>
         {
             using type = typename std::decay_t<E>::expression_tag;
         };
 
+        template <class E>
+        struct get_expression_tag : get_expression_tag_impl<E>
+        {
+        };
+
         template <class E>
         using get_expression_tag_t = typename get_expression_tag<E>::type;
 
         template <class... T>
         struct expression_tag_and;
 
+        template <>
+        struct expression_tag_and<>
+        {
+            using type = xtensor_expression_tag;
+        };
+
         template <class T>
         struct expression_tag_and<T>
         {
@@ -258,34 +322,22 @@ namespace xt
             using type = T;
         };
 
-        template <>
-        struct expression_tag_and<xscalar_expression_tag, xscalar_expression_tag>
-        {
-            using type = xscalar_expression_tag;
-        };
-
         template <class T>
-        struct expression_tag_and<xscalar_expression_tag, T>
+        struct expression_tag_and<xtensor_expression_tag, T>
         {
             using type = T;
         };
 
         template <class T>
-        struct expression_tag_and<T, xscalar_expression_tag>
-            : expression_tag_and<xscalar_expression_tag, T>
+        struct expression_tag_and<T, xtensor_expression_tag>
+            : expression_tag_and<xtensor_expression_tag, T>
         {
         };
 
         template <>
-        struct expression_tag_and<xtensor_expression_tag, xoptional_expression_tag>
-        {
-            using type = xoptional_expression_tag;
-        };
-
-        template <>
-        struct expression_tag_and<xoptional_expression_tag, xtensor_expression_tag>
-            : expression_tag_and<xtensor_expression_tag, xoptional_expression_tag>
+        struct expression_tag_and<xtensor_expression_tag, xtensor_expression_tag>
         {
+            using type = xtensor_expression_tag;
         };
 
         template <class T1, class... T>
@@ -296,12 +348,17 @@ namespace xt
 
         template <class... T>
         using expression_tag_and_t = typename expression_tag_and<T...>::type;
+
+        struct xtensor_empty_base
+        {
+            using expression_tag = xtensor_expression_tag;
+        };
     }
 
     template <class... T>
     struct xexpression_tag
     {
-        using type = detail::expression_tag_and_t<detail::get_expression_tag_t<std::decay_t<const_xclosure_t<T>>>...>;
+        using type = extension::expression_tag_and_t<extension::get_expression_tag_t<std::decay_t<const_xclosure_t<T>>>...>;
     };
 
     template <class... T>
