@@ -1332,14 +1332,12 @@ namespace xt
     template <class T>
     inline auto xkeep_slice<T>::step_size(std::size_t i, std::size_t n) const noexcept -> size_type
     {
-        // special case one-past-end step (should be removed soon)
-        if (i == m_indices.size())
+        if (i + n >= m_indices.size())
         {
-            return 1;
+            return m_indices.back() - m_indices[i] + 1;
         }
         else
         {
-            --i;
             return m_indices[i + n] - m_indices[i];
         }
     }
@@ -1476,15 +1474,13 @@ namespace xt
     template <class T>
     inline auto xdrop_slice<T>::step_size(std::size_t i, std::size_t n) const noexcept -> size_type
     {
-        // special case one-past-end step (should be removed soon)
-        if(i == static_cast<std::size_t>(m_size))
+        if (i + n >= static_cast<std::size_t>(m_size))
         {
-            return 1;
+            return (*this)(static_cast<size_type>(m_size-1)) - (*this)(static_cast<size_type>(i)) + 1;
         }
         else
         {
-            --i;
-            return (*this)(i + n) - (*this)(i);
+            return (*this)(static_cast<size_type>(i + n)) - (*this)(static_cast<size_type>(i));
         }
     }
 
